using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Data;

namespace ADSUtil
{
	public delegate void PinClickHandler(Pin source, PinClickEventArgs args);

	public enum PinType
	{
		Analog = 0,
		Digital = 1
	}

	public enum PinAlignment
	{
		PinOnLeft = 0,
		PinOnRight = 1
	}

	public class PinClickEventArgs : EventArgs 
	{
		// a PinClick passes the Pin Number and the Pin's Text
		public int number;
		public string text;

		public PinClickEventArgs(int PinNumber, string PinText) 
		{
			number = PinNumber;
			text = PinText;
		}
	}

	public class Pins : CollectionBase
	{
		public void Add(Pin PinToAdd)
		{
			List.Add(PinToAdd);
		}

		public void Remove(Pin PinToRemove)
		{
			List.Remove(PinToRemove);
		}

		// Indexer for Pins
		public Pin this[byte Index]
		{
			get
			{
				return (Pin)List[Index];
			}
			set
			{
				List[Index] = value;
			}
		}

		public Pins(){}

	}
	public class Pin : System.Windows.Forms.Control
	{
		public event PinClickHandler PinClick;
		
		#region general Pin settings
		private Size	pinSize			= new Size(10, 10);
		private Size	controlSize		= new Size(85, 20);
		private Color	digitalOnColor	= Color.Orange;
		private Color	digitalOffColor = Color.Blue;
		private Color	analogColor		= Color.Green;
		private Color	disabledColor	= Color.Gray;
		#endregion

		#region local variables
		PinType			type;
		uint			pinValue;
		Rectangle		pinBounds;
		Rectangle		pinBorder;
		Rectangle		textBounds;
		bool			midClick;
		int				pinNumber;
		PinAlignment	align;
		bool			showValue;
		#endregion

		#region Properties
		public PinAlignment Alignment
		{ // determines where the pin rectange is placed
			set
			{
				align = value;
				
				if(value == PinAlignment.PinOnRight)
				{
					this.pinBorder = new Rectangle(
						this.ClientRectangle.Width - (pinSize.Width + 10),
						1,
						pinSize.Width + 9,
						this.ClientRectangle.Height - 2);

					this.pinBounds = new Rectangle(
							this.ClientRectangle.Width - (pinSize.Width + 5), 
							((this.ClientRectangle.Height - pinSize.Height)) / 2 + 1, 
							pinSize.Width, 
							pinSize.Height);
					
					this.textBounds = new Rectangle(
							5, 
							5, 
							this.ClientRectangle.Width - (pinSize.Width + 10), 
							20);
				}
				else
				{
					this.pinBorder = new Rectangle(
						1,
						1,
						pinSize.Width + 9,
						this.ClientRectangle.Height - 2);

					this.pinBounds = new Rectangle(
							6, 
							this.ClientRectangle.Height - (pinSize.Height + 4), 
							pinSize.Width, 
							pinSize.Height);

					this.textBounds = new Rectangle(
							pinSize.Width + 10,
							5,
							this.ClientRectangle.Width - (pinSize.Width + 10),
							20);
				}

				this.Invalidate();
			}
			get
			{
				return align;
			}
		}

		public bool DisplayValue
		{
			set
			{
				showValue = value;
				this.Invalidate(textBounds);
			}
			get
			{
				return showValue;
			}
		}

		public PinType Type
		{
			set
			{
				type = value;
				this.Invalidate();
			}
			get
			{
				return type;
			}
		}

		public int PinNumber
		{
			set
			{
				// save the new value
				pinNumber = value;
			}
			get
			{
				return pinNumber;
			}
		}

		public uint Value
		{
			set
			{
				// update only if there's a change
				if(pinValue != value)
				{

					// save the new value
					pinValue = value;

					// redraw me
					this.Invalidate(pinBorder);

					if(showValue)
						this.Invalidate(textBounds);
				}
			}
			get
			{
				return pinValue;
			}
		}
		#endregion

		#region contructors
		public Pin(PinType Type, uint InitialValue) : this()
		{			
			pinValue = InitialValue;
			type = Type;
		}

		public Pin(PinType Type, PinAlignment Alignment, uint InitialValue) : this()
		{
			pinValue = InitialValue;
			this.Alignment = Alignment;
			type = Type;
		}

		public Pin(PinType Type, PinAlignment Alignment) : this()
		{
			this.Alignment = Alignment;
			type = Type;
		}

		public Pin(PinType Type) : this()
		{			
			type = Type;
		}

		public Pin()
		{
			// all ctors call this base ctor
			InitializeComponent();

			this.MouseDown += new MouseEventHandler(PinMouseDown);
			this.MouseUp += new MouseEventHandler(PinMouseUp);
			
			showValue = false;
			pinValue = 0;
			type = PinType.Digital;
			Alignment = PinAlignment.PinOnRight;
		}
		#endregion

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Size = controlSize;
		}

		#endregion

		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}

		private void PinMouseDown(object sender, MouseEventArgs e)
		{
			if(!this.Enabled)
				return;
			
			// if the user clicked in the "pin" rectangle, start a click process
			midClick = pinBorder.Contains(e.X, e.Y);				
		}

		private void PinMouseUp(object sender, MouseEventArgs e)
		{
			// if we had a mousedown and then up inside the "pin" rectangle, fire a click
			if((midClick) && (pinBorder.Contains(e.X, e.Y)))
			{
				if(PinClick != null)
					PinClick(this, new PinClickEventArgs(this.PinNumber, this.Text));
			}
		}

		protected override void OnPaint(PaintEventArgs pe)
		{
			Brush b;

			if(this.Enabled)
			{
				if(type == PinType.Digital)
				{
					// digital pins have different on/off color
					b = new System.Drawing.SolidBrush(this.Value == 0 ? (digitalOffColor) : (digitalOnColor));
				}
				else
				{
					// analog pin
					b = new System.Drawing.SolidBrush(analogColor);
				}

			}
			else
			{
				// disabled pin
				b = new System.Drawing.SolidBrush(disabledColor);
			}

			// draw the pin
			if(this.PinNumber == 1)
				pe.Graphics.FillRectangle(b, pinBounds);
			else
				pe.Graphics.FillEllipse(b, pinBounds);

			// draw a border Rectangle around the pin
			pe.Graphics.DrawRectangle(new Pen(Color.Black), pinBorder);

			// draw the text centered in the text bound
			string drawstring;

			// are we showing the Text or Value?
			if(showValue)
				drawstring = Convert.ToString(this.Value);
			else
				drawstring = this.Text;
			
			// determine the actual string size
			SizeF fs = pe.Graphics.MeasureString(
						drawstring, 
						new Font(FontFamily.GenericMonospace, 8f, FontStyle.Regular));

			// draw the string
			pe.Graphics.DrawString(
						drawstring, 
						new Font(FontFamily.GenericMonospace, 8f, FontStyle.Regular), 
						new SolidBrush((showValue ? analogColor : Color.Black)),
						textBounds.X + (textBounds.Width - fs.ToSize().Width) / 2,
						textBounds.Y + (textBounds.Height - fs.ToSize().Height) / 2);

			// clean up the Brush
			b.Dispose();
		}     	
	}

	public class Connector : System.Windows.Forms.Control
	{
		public event PinClickHandler PinClick;
		protected Pins pins;
		byte pincount;

		public Connector(byte TotalPins)
		{
			pins = new Pins();
			pincount = TotalPins;

			InitializeComponent();
		}

		#region Component Designer generated code
		private void InitializeComponent()
		{
			for(int i = 0 ; i < pincount ; i++)
			{
				Pin p = new Pin(PinType.Digital, (PinAlignment)((i + 1) % 2), 0);
				p.PinClick += new PinClickHandler(OnPinClick);
				p.PinNumber = i + 1;
				p.Text = Convert.ToString(i);
				p.Top = (i / 2) * p.Height;
				p.Left = (i % 2) * p.Width;
				this.Pins.Add(p);

				this.Controls.Add(p);
			}

			this.Width = Pins[0].Width * 2;
			this.Height = Pins[0].Height * this.Pins.Count / 2;
		}
		#endregion

		public Pins Pins
		{
			set
			{
				pins = value;
			}
			get
			{
				return pins;
			}
		}

		private void OnPinClick(Pin sender, PinClickEventArgs e)
		{
			// pass on the event
			if(PinClick != null) PinClick(sender, e);

			// handle click
//			if(sender.Type == PinType.Digital)
//				sender.Value = sender.Value == 0 ? 1 : 0;
//			else
//				sender.DisplayValue = !sender.DisplayValue;

		}

		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
	}

	public class GMasterJ2 : Connector
	{
		public enum PinName : byte
		{
			UCB_IO0 = 1,
			UCB_IO9,
			UCB_IO1,
			UCB_IO8,
			UCB_IO2,	//5
			UCB_IO7,
			UCB_IO3,
			UCB_IO6,
			UCB_IO4,
			UCB_IO5,	//10
			ANIN0,
			PB0,
			ANIN1,
			PB1,
			ANIN2,		//15
			PB2,
			ANIN3,
			PB3,
			UCB_TINP,
			PD0,		//20
			UCB_TINN,
			PD1,
			UCB_TOUTN,
			EXT_IRQ,
			UCB_TOUTP,	//25
			USB_VCC,
			USB_GND,
			USB_UDCN,
			GND,
			USB_UDCP, //30
		}

		public GMasterJ2() : base(30)
		{
			foreach(Pin p in this.Pins)
			{
				PinName pn = (PinName)p.PinNumber;
				p.Text = pn.ToString();
			}

			this.Pins[(byte)(PinName.ANIN0 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.ANIN1 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.ANIN2 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.ANIN3 - 1)].Type = PinType.Analog;

			this.Pins[(byte)(PinName.EXT_IRQ - 1)].Enabled = false;
			this.Pins[(byte)(PinName.UCB_TINN - 1)].Enabled = false;
			this.Pins[(byte)(PinName.UCB_TINP - 1)].Enabled = false;
			this.Pins[(byte)(PinName.UCB_TOUTN - 1)].Enabled = false;
			this.Pins[(byte)(PinName.UCB_TOUTP - 1)].Enabled = false;
			this.Pins[(byte)(PinName.USB_GND - 1)].Enabled = false;
			this.Pins[(byte)(PinName.USB_UDCN - 1)].Enabled = false;
			this.Pins[(byte)(PinName.USB_UDCP - 1)].Enabled = false;
			this.Pins[(byte)(PinName.USB_VCC - 1)].Enabled = false;
			this.Pins[(byte)(PinName.GND - 1)].Enabled = false;
		}

		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
	}

	public class GMasterJ7 : Connector
	{
		public enum PinName : byte
		{
			PC7 = 1,
			RXD2,
			PC6,
			TXD2,
			PC5,	//5
			CTS2,
			PC4,
			RTS2,
			PC3,
			GNDCOM2, //10
			PC2,
			RXD1,
			PC1,
			TXD1,
			PC0,	//15
			CTS1,
			PA0,
			RTS1,
			PA1,
			GNDCOM1, //20
			PA2,
			RX422P,
			PA3,
			RX422N,
			PA4,	//25
			TX422P,
			PA5,
			TX422N,
			PA6,
			GNDRS422, //30
			PA7,
			LED1,
			VREF,
			LED2,
			VCC,	//35
			C7,
			V33,
			C6,
			GND1,
			GND2	//40
		}

		public GMasterJ7() : base(40)
		{
			foreach(Pin p in this.Pins)
			{
				PinName pn = (PinName)p.PinNumber;
				p.Text = pn.ToString();
			}

			#region Uncomment for Analog SIO input (Option 2)
			/*
			this.Pins[(byte)(PinName.PA0 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA1 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA2 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA3 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA4 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA5 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA6 - 1)].Type = PinType.Analog;
			this.Pins[(byte)(PinName.PA7 - 1)].Type = PinType.Analog;		
			*/
			#endregion

			this.Pins[(byte)(PinName.GND1 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.GND2 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.V33 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.VCC - 1)].Enabled = false;
			this.Pins[(byte)(PinName.VREF - 1)].Enabled = false;
			this.Pins[(byte)(PinName.RX422N - 1)].Enabled = false;
			this.Pins[(byte)(PinName.RX422P - 1)].Enabled = false;
			this.Pins[(byte)(PinName.TX422N - 1)].Enabled = false;
			this.Pins[(byte)(PinName.TX422P - 1)].Enabled = false;
			this.Pins[(byte)(PinName.GNDCOM1 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.GNDCOM2 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.GNDRS422 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.RTS1 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.RTS2 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.CTS1 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.CTS2 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.C6 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.C7 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.RXD1 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.TXD1 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.RXD2 - 1)].Enabled = false;
			this.Pins[(byte)(PinName.TXD2 - 1)].Enabled = false;
		}
	}

}
